#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class remove_stuck_particles(Boolean):
    """
    Enable/disable removal of particles that are stuck at edges or faces.
    """

    fluent_name = "remove-stuck-particles?"

