#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

class set_maximum_dynamic_strength(Real):
    """
    Set maximum value of dynamic anti-diffusion strength.
    """

    fluent_name = "set-maximum-dynamic-strength"

